/*
 *  Copyright 2002-2004, Mersad Team, Allame Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  Created by: Ahmad Boorghany
 *  Released on Friday 1 April 2005 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#ifndef __GROUP_WORKS_H
#define __GROUP_WORKS_H

#include <SayDecision.h>
#include <Positioning.h>
#include <VirtualPass.h>
#include <HeadDecision.h>

enum WithoutBallSelection
{
	WOBS_BLOCK,
	WOBS_INTERCEPT,
	WOBS_POSITIONING
};

class GroupWorks
{
protected:
	bool interceptSay;
	bool interceptBlock;
	bool lastWithBallDecideValidFlag;
	unsigned interceptNum;
	unsigned lastPassHearTime;

	unsigned requestNum;
	unsigned lastRequestNum;
	unsigned hearedRequestPassNum;
	unsigned hearedRequestPassTime;
	unsigned hearedRequestPassSender;

	const WorldModel *worldModel;
	const Positioning &positioning;

	SayDecisionFlags &sayFlags;
	TurnNeckMode &turnNeckMode;
	VirtualKickCalculate virtualKickCalc;

	VirtualPass virtualPass;

	float getBlockWeight(const Point &ballStopPoint,
			const Player &player) const;

public:
	GroupWorks(const WorldModel *worldModel, SayDecisionFlags &sayFlags,
			TurnNeckMode &turnNeckMode, const Positioning &positioning);

	bool getChangeInterceptNumPermission() const;
	void updateInterceptNum();
	void setInterceptNum(unsigned uniNum);
	void resetByChangePlayMode();
	void changeInterceptNumByHear(unsigned sender, unsigned uniNum);
	unsigned getInterceptNumForSay();

	void checkForLastWithBallDecideValid();

	bool isLastWithBallDecideValid() const;
	WithoutBallSelection getWithoutBallSelection(
			DecideType lastWithBallDecision) const;

	void setLastWithBallDecideValid(bool lastWithBallDecisionValidFlagArg);

	void suggestPassSayManager();
	void requestSuggestPassSayManager();

	bool getPassSayRequestPermittion();

	void updateAfterSenseBody();

	void decide();

	// setting functions
	void setHearedRequestPassNum(unsigned hearedRequestPassNumArg);
	void setHearedRequestPassTime(unsigned hearedRequestPassTimeArg);
	void setHearedRequestPassSender(unsigned hearedRequestPassNumSenderArg);

};


#endif // __GROUP_WORKS_H

