/*
 *  Copyright 2002-2004, Mersad Team, Allame Helli High School (NODET).
 *
 *  This program is free software, you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU Library General Public License for more details.
 *
 *  Created by: Ahmad Boorghany
 *  Released on Friday 1 April 2005 by Mersad RoboCup Team.
 *  For more information please read README file.
*/

#ifndef __HEAD_DECISION_H
#define __HEAD_DECISION_H


#include <WorldModel.h>


#define TURN_NECK_RESOLUTION 5


enum TurnNeckMode {
	TNM_NONE,
	TNM_AUTO_PLAY_ON,
	TNM_LOOK_NORMALLY,
	TNM_LOOK_CAREFULLY,
	TNM_LOOK_NORMALLY_TO_BALL,
	TNM_LOOK_CAREFULLY_TO_BALL,
	TNM_LOOK_CAREFULLY_TO_OPP_GOALIE
};

class Command;
class AdvancedAgent;

class HeadDecision
{
protected:
	float viewPartWeights[VIEW_PARTS_NUM];
	unsigned viewPartObjectsNums[VIEW_PARTS_NUM];

	Body nextBody;
	Ball nextBall;
	TurnNeckMode mode;
	Command *turnNeckCommand;
	Command *changeViewCommand;

	// Weighting rates
	float haveBallRate;
	float havePlayerRate;
	float haveOppGoalRate;
	float objectsNumRate;

	// Giving vars
	const AdvancedAgent *owner;
	const WorldModel *worldModel;

	void decideForLookCarefullyToBall();
	void decideForLookCarefullyToOppGoalie();
	void decideForTurnNeckCommand(float nextViewAngle);
	void decideForLookNormallyToBall(float nextViewAngle);

	void logVariables();
	void updateViewPartObjectsNum();
	void setWeightsForLookNormally();
	void setWeightsForLookCarefully();

	float getViewPartWeight(unsigned viewPartNum);

public:
	HeadDecision(const AdvancedAgent *owner);
	~HeadDecision();

	void resetInfo();
	void decide(const Command *bodyCycleCommand);

	void updateAfterSee();
	void updateAfterSenseBody();

	Command *getTurnNeckCommand();
	Command *getChangeViewCommand();

	TurnNeckMode &getModeRef() const;

	// Setting functions
	void setMode(TurnNeckMode modeArg);
};

#endif // __HEAD_DECISION_H

